@@ -1,22 +1,19 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2019 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Quadratic constraint types."""
@@ -106,9 +103,9 @@ class NonconvexQuadraticConstraint(Constraint):
 
     def _get_slack(self):
         if self.relation == self.LE:
-            return self.rhs.value - self.lhs.value
+            return self.rhs.safe_value - self.lhs.safe_value
         else:
-            return self.lhs.value - self.rhs.value
+            return self.lhs.safe_value - self.rhs.safe_value
 
 
 class ConicQuadraticConstraint(NonconvexQuadraticConstraint):
@@ -140,7 +137,7 @@ class ConicQuadraticConstraint(NonconvexQuadraticConstraint):
     <Conic Quadratic Constraint: x² + 1 ≤ y·z>
     >>> P = C.__class__.Conversion.convert(C, Options(assume_conic=True))
     >>> list(P.constraints.values())[0]
-    <4×1 RSOC Constraint: ‖fullroot(x² + 1)‖² ≤ y·z ∧ y ≥ 0>
+    <4×1 RSOC Constraint: ‖fullroot(x² + 1)‖² ≤ y·z ∧ y, z ≥ 0>
     >>> Q = C.__class__.Conversion.convert(C, Options(assume_conic=False))
     >>> list(Q.constraints.values())[0]
     <Nonconvex Quadratic Constraint: x² + 1 ≤ y·z>
