@@ -1,32 +1,30 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2019-2020 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Domain-specific container types."""
 
 from collections import OrderedDict
+from collections.abc import MutableSet
 from itertools import chain
+from types import MappingProxyType
 
 from .apidoc import api_end, api_start
 from .caching import cached_property
-from .compat import MappingProxyType, MutableSet
 
 _API_START = api_start(globals())
 # -------------------------------
@@ -261,7 +259,7 @@ class RecordTreeToken:
             self.__class__.__name__))
 
 
-class RecordTree(object):
+class RecordTree():
     """Labeled tree for storing records.
 
     An immutable labeled tree with values at the leaf nodes, where labels and
@@ -325,9 +323,8 @@ class RecordTree(object):
         try:
             hash(newValue)
         except Exception as error:
-            raise TypeError(
-                "Failed to freeze {} to a hashable type: Hashing {} gives: {}"
-                .format(value, newValue, error))
+            raise TypeError("Failed to freeze {} to a hashable type."
+                .format(value)) from error
 
         return newValue
 
@@ -484,10 +481,6 @@ class RecordTree(object):
     def __eq__(self, other):
         return hash(self) == hash(other)
 
-    def __ne__(self, other):
-        # Python 2 needs this in addition to __eq__.
-        return hash(self) != hash(other)
-
     def __bool__(self):
         return bool(self._tree)
 
@@ -611,6 +604,38 @@ class RecordTree(object):
 
         return True
 
+    def mismatch(self, other):
+        """A subtree of ``self`` that renders ``self << other`` :obj:`False`.
+
+        :returns RecordTree:
+            The smallest subtree ``T`` of ``self`` such that ``self`` without
+            the records in ``T`` is a subtree of ``other``. The returned tree is
+            a direct instance of the :class:`RecordTree` base class.
+        """
+        if not isinstance(other, RecordTree):
+            raise TypeError("The argument must be another record tree.")
+
+        records = []
+
+        for record in self.records:
+            lhsNode = self._tree
+            rhsNode = other._tree
+
+            for label in record[:-1]:
+                if rhsNode is self.ALL:
+                    break
+
+                if label not in rhsNode:
+                    records.append(record)
+                    break
+
+                assert label in lhsNode
+
+                lhsNode = lhsNode[label]
+                rhsNode = rhsNode[label]
+
+        return RecordTree(records)
+
     @staticmethod
     def _str(thing):
         return thing.__name__ if hasattr(thing, "__name__") else str(thing)
