@@ -1,23 +1,20 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2019 Maximilian Stahlberg
 # Based on the original picos.expressions module by Guillaume Sagnol.
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Implements :class:`Entropy` and :class:`NegativeEntropy`."""
@@ -164,11 +161,11 @@ class Entropy(Expression):
         return cvxopt.matrix(s)
 
     @cached_unary_operator
-    def _get_variables(self):
+    def _get_mutables(self):
         if self._y is None:
-            return self._x.variables
+            return self._x._get_mutables()
         else:
-            return self._x.variables.union(self._y.variables)
+            return self._x._get_mutables().union(self._y.mutables)
 
     def _is_convex(self):
         return False
@@ -176,9 +173,13 @@ class Entropy(Expression):
     def _is_concave(self):
         return True
 
-    def _replace_variables(self, var_map):
-        return self.__class__(self._x._replace_variables(var_map),
-            None if self._y is None else self._y._replace_variables(var_map))
+    def _replace_mutables(self, mapping):
+        return self.__class__(self._x._replace_mutables(mapping),
+            None if self._y is None else self._y._replace_mutables(mapping))
+
+    def _freeze_mutables(self, freeze):
+        return self.__class__(self._x._freeze_mutables(freeze),
+            None if self._y is None else self._y._freeze_mutables(freeze))
 
     # --------------------------------------------------------------------------
     # Python special method implementations, except constraint-creating ones.
@@ -400,11 +401,11 @@ class NegativeEntropy(Expression):
         return cvxopt.matrix(s)
 
     @cached_unary_operator
-    def _get_variables(self):
+    def _get_mutables(self):
         if self._y is None:
-            return self._x.variables
+            return self._x._get_mutables()
         else:
-            return self._x.variables.union(self._y.variables)
+            return self._x._get_mutables().union(self._y.mutables)
 
     def _is_convex(self):
         return True
@@ -412,9 +413,13 @@ class NegativeEntropy(Expression):
     def _is_concave(self):
         return False
 
-    def _replace_variables(self, var_map):
-        return self.__class__(self._x._replace_variables(var_map),
-            None if self._y is None else self._y._replace_variables(var_map))
+    def _replace_mutables(self, mapping):
+        return self.__class__(self._x._replace_mutables(mapping),
+            None if self._y is None else self._y._replace_mutables(mapping))
+
+    def _freeze_mutables(self, freeze):
+        return self.__class__(self._x._freeze_mutables(freeze),
+            None if self._y is None else self._y._freeze_mutables(freeze))
 
     # --------------------------------------------------------------------------
     # Python special method implementations, except constraint-creating ones.
