@@ -1,22 +1,19 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2020 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Implements :class:`SquaredNorm`."""
@@ -84,7 +81,7 @@ class SquaredNorm(QuadraticExpression):
             vec = x.vec if x.isreal else x.vec.real // x.vec.imag
 
             # Remove zero rows from the vectorization.
-            A = abs(cvxopt_hcat(list(vec._coefs.values()) + [vec._const]))
+            A = abs(cvxopt_hcat(vec._coefs.values()))
             a = cvxopt.sparse(sum(A[:, j] for j in range(A.size[1])))
             nonzero = a.I
             nnz = len(nonzero)
@@ -252,11 +249,6 @@ class SquaredNorm(QuadraticExpression):
         else:
             return QuadraticExpression.__truediv__(self, other)
 
-    def __div__(self, other):
-        """Denote division by a scalar constant."""
-        # Python 2 uses this instead of __truediv__.
-        return self.__truediv__(other)
-
     # --------------------------------------------------------------------------
     # Method overridings for QuadraticExpression.
     # --------------------------------------------------------------------------
@@ -265,8 +257,8 @@ class SquaredNorm(QuadraticExpression):
     def fullroot(self):
         """Affine expression whose squared norm equals the expression.
 
-        Overrides :meth:`~picos.expressions.QuadraticExpression.fullroot` of
-        :class:`~picos.expressions.QuadraticExpression`.
+        Overrides :meth:`~.exp_quadratic.QuadraticExpression.fullroot` of
+        :class:`~.exp_quadratic.QuadraticExpression`.
         """
         return self._x.renamed(glyphs.Fn("fullroot({})")(self.string))
 
@@ -274,8 +266,8 @@ class SquaredNorm(QuadraticExpression):
     def is_squared_norm(self):
         """Always :obj:`True` for squared norm instances.
 
-        Overrides :meth:`~picos.expressions.QuadraticExpression.is_squared_norm`
-        of :class:`~picos.expressions.QuadraticExpression`.
+        Overrides :meth:`~.exp_quadratic.QuadraticExpression.is_squared_norm`
+        of :class:`~.exp_quadratic.QuadraticExpression`.
         """
         return True
 
@@ -283,8 +275,8 @@ class SquaredNorm(QuadraticExpression):
     def is0(self):
         """Whether the expression is zero.
 
-        Overrides :meth:`~picos.expressions.QuadraticExpression.is0` of
-        :class:`~picos.expressions.QuadraticExpression`.
+        Overrides :meth:`~.exp_quadratic.QuadraticExpression.is0` of
+        :class:`~.exp_quadratic.QuadraticExpression`.
         """
         return self._x.is0
 
