@@ -1,28 +1,23 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2019 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Console output helpers related to formatting."""
 
-from __future__ import print_function
-
 import re
 import sys
 from contextlib import contextmanager
@@ -450,5 +445,35 @@ def parameterized_string(
     return template, data
 
 
+def arguments(strings, sep=", ", empty=""):
+    """A wrapper around :func:`parameterized_string` for argument lists.
+
+    :param list(str) strings:
+        String descriptions of the arguments.
+
+    :param str sep:
+        Separator.
+
+    :param str empty:
+        Representation of an empty argument list.
+    """
+    if len(strings) == 0:
+        return empty
+    elif len(strings) == 1:
+        return strings[0]
+    elif len(strings) == 2:
+        return strings[0] + sep + strings[1]
+
+    try:
+        template, data = parameterized_string(strings)
+    except ValueError:
+        pass
+    else:
+        if data:
+            return glyphs.sep(template, data)
+
+    return glyphs.fromto(strings[0] + sep, sep + strings[-1])
+
+
 # --------------------------------------
 __all__ = api_end(_API_START, globals())
