@@ -1,22 +1,19 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2019-2020 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Optimization solver parameter handling."""
@@ -29,7 +26,7 @@ import fnmatch
 import types
 
 from ..apidoc import api_end, api_start
-from ..solvers import all_solvers
+from ..solvers import all_solvers, Solver
 
 _API_START = api_start(globals())
 # -------------------------------
@@ -41,9 +38,8 @@ OPTIONS = [
 
     ("strict_options", bool, False, """
         Whether unsupported general options will raise an
-        :class:`UnsupportedOptionError
-        <picos.solvers.solver.UnsupportedOptionError>`
-        exception, instead of printing a warning."""),
+        :class:`~.solver.UnsupportedOptionError` exception, instead of printing
+        a warning."""),
 
     ("verbosity", int, 0, """
         Verbosity level.
@@ -58,26 +54,39 @@ OPTIONS = [
         Whether solvers are allowed to ignore the :ref:`verbosity
         <option_verbosity>` option to print licensing related warnings.
 
-        .. warning::
-
-            Using this option to suppress licensing related warnings is done at
-            your own legal responsibility."""),
+        See also the global setting :data:`~.settings.LICENSE_WARNINGS`.
+        """),
 
     ("solver", str, None, """
         The solver to use.
 
+        See also the global settings :data:`~.settings.SOLVER_BLACKLIST`,
+        :data:`~.settings.SOLVER_WHITELIST` and
+        :data:`~.settings.NONFREE_SOLVERS`.
+
         - :obj:`None` to let PICOS choose.
         - """ + """
         - """.join('``"{0}"`` for :class:`{1} <picos.solvers.{1}>`.'
             .format(name, solver.__name__)
             for name, solver in all_solvers().items()) + """
 
+        This option is ignored when :ref:`ad_hoc_solver <option_ad_hoc_solver>`
+        is set.
+
         .. note::
 
             :func:`picos.available_solvers() <picos.available_solvers>` returns
             a list of names of solvers that are available at runtime.
         """, lambda value: value is None or value in all_solvers().keys()),
 
+    ("ad_hoc_solver", type, None, """
+        The solver to use as a :class:`~.solvers.solver.Solver` subclass.
+
+        This allows solver implementations to be shipped independent of PICOS.
+
+        If set, takes precedence over :ref:`solver <option_solver>`.""",
+        lambda value: value is None or issubclass(value, Solver)),
+
     ("primals", bool, True, """
         Whether to request a primal solution.
 
@@ -104,7 +113,7 @@ OPTIONS = [
         Whether to dualize the problem as part of the solution strategy.
 
         This can sometimes lead to a significant solution search speedup.
-     """),
+        """),
 
     ("assume_conic", bool, True, r"""
         Determines how :class:`~picos.constraints.ConicQuadraticConstraint`
@@ -132,7 +141,8 @@ OPTIONS = [
 
         If multiple solutions are returned by the solver, then the first one
         will be applied. If this is ``False``, then solutions can be applied
-        manually via their :func:`apply <picos.Solution.apply>` method. """),
+        manually via their :meth:`~.solution.Solution.apply` method.
+        """),
 
     ("abs_prim_fsb_tol", float, 1e-8, """
         Absolute primal problem feasibility tolerance.
@@ -304,8 +314,8 @@ OPTIONS = [
     ("pool_size", int, None, """
         Maximum number of mixed integer feasible solutions returned.
 
-        If this is not :obj:`None`, :func:`Problem.solve <picos.Problem.solve>`
-        returns a list of :class:`solutions <picos.Solution>` instead of just a
+        If this is not :obj:`None`, :meth:`~.problem.Problem.solve`
+        returns a list of :class:`~.solution.Solution` objects instead of just a
         single one.
 
         :obj:`None` lets the solver return only the best solution.
@@ -375,7 +385,7 @@ OPTIONS = [
         during the MIP solution search process. At the end of the computation, a
         list of triples ``(time, lowerbound, upperbound)`` will be provided in
         the field ``bounds_monitor`` of the dictionary returned by
-        :func:`solve <picos.Problem.solve>`.
+        :meth:`~.problem.Problem.solve`.
         """),
 
     ("cvxopt_kktsolver", (str, types.FunctionType), "ldl", """
@@ -458,15 +468,15 @@ for name, solver in all_solvers().items():
         :math:`10^x` times larger as when passed to :math:`B`.
         """.format(name.upper())))
 
+del name, solver
+
 OPTIONS = sorted(OPTIONS)
 
 
-# Explicit object inheritence is necessary for __new__ under Python 2.
-class Option(object):
+class Option():
     """Optimization solver option.
 
-    A single option that affects how a :class:`Problem <picos.modeling.Problem>`
-    is solved.
+    A single option that affects how a :class:`~.problem.Problem` is solved.
 
     An initial instance of this class is built from each entry of the
     :data:`OPTIONS` table to obtain the :data:`OPTION_OBJS` tuple.
@@ -507,21 +517,19 @@ class Option(object):
             if isinstance(self.argType, type):
                 try:
                     value = self.argType(value)
-                except Exception:
+                except Exception as error:
                     raise TypeError("Failed to convert argument {} to option "
-                        "'{}' to type {}.".format(
-                        repr(value), self.name, self.argType.__name__))
+                        "'{}' to type {}.".format(repr(value), self.name,
+                        self.argType.__name__)) from error
             else:
                 assert isinstance(self.argType, (tuple, list))
                 assert all(isinstance(t, type) for t in self.argType)
+
                 raise TypeError("Argument {} to option '{}' does not match "
                     "permissible types {}.".format(repr(value), self.name,
                     ", ".join(t.__name__ for t in self.argType)))
 
-        try:
-            if self.check is not None and not self.check(value):
-                raise ValueError()
-        except Exception:
+        if self.check is not None and not self.check(value):
             raise ValueError("The option '{}' does not accept the value {}."
                 .format(self.name, repr(value)))
 
@@ -571,13 +579,12 @@ def _jumplabel(option):
     return ":ref:`{0} <option_{0}>`".format(option.name)
 
 
-# Explicit object inheritence is necessary for __new__ under Python 2.
-class Options(object):
+class Options():
     """Collection of optimization solver options.
 
-    A collection of options that affect how a :class:`Problem
-    <picos.modeling.Problem>` is solved. :attr:`Problem.options
-    <picos.Problem.options>` is an instance of this class.
+    A collection of options that affect how a :class:`~.problem.Problem` is
+    solved. :attr:`Problem.options <.problem.Problem.options>` is an instance of
+    this class.
 
     The options can be accessed as an attribute or as an item. The latter
     approach supports Unix shell-style wildcard characters:
@@ -750,11 +757,8 @@ class Options(object):
 
     def __dir__(self):
         optionNames = [name for name in self._options.keys()]
-        if hasattr(super(Options, self), "__dir__"):  # Python 3
-            list = super(Options, self).__dir__() + optionNames
-        else:  # Python 2
-            list = dir(self.__class__) + self.__dict__.keys() + optionNames
-        return sorted(list)
+        list_ = super(Options, self).__dir__() + optionNames
+        return sorted(list_)
 
     def copy(self):
         """Return an independent copy of the current options set."""
