@@ -1,22 +1,19 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2019 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Optimization problem solution representation."""
@@ -129,6 +126,7 @@ def _check_type(argument, *types):
             for t in types)))
 
 
+# TODO: Make all public fields use snake_case, ensure backwards compatibility.
 class Solution:
     """Assignment of primal and dual values to variables and constraints.
 
@@ -160,7 +158,7 @@ class Solution:
     def __init__(self, primals, duals=None, problem=None, solver="user",
             primalStatus=SS_UNKNOWN, dualStatus=SS_UNKNOWN,
             problemStatus=PS_UNKNOWN, searchTime=0.0, info=None,
-            vectorizedPrimals=False):
+            vectorizedPrimals=False, reportedValue=None):
         """Create a solution to an optimization problem.
 
         :param dict(picos.expressions.BaseVariable, object) primals:
@@ -185,11 +183,16 @@ class Solution:
         :param bool vectorizedPrimals:
             Whether primal solution values are given with respect to the
             variable's special vectorization format as used by PICOS internally.
+        :param float reportedValue:
+            Objective value of the solution as reported by the solver.
         """
         from ..expressions import BaseVariable
         from ..constraints import Constraint
         from .problem      import Problem
 
+        if primals is None:
+            primals = {}
+
         if duals is None:
             duals = {}
 
@@ -198,8 +201,8 @@ class Solution:
 
         # Be strict about the arguments as they are handed to the user.
         _check_type(primals, dict)
-        _check_type(problem, None, Problem)
         _check_type(duals, dict)
+        _check_type(problem, None, Problem)
         _check_type(solver, str)
         _check_type(primalStatus, str)
         _check_type(dualStatus, str)
@@ -207,6 +210,7 @@ class Solution:
         _check_type(searchTime, float)
         _check_type(info, dict)
         _check_type(vectorizedPrimals, bool)
+        _check_type(reportedValue, None, float)
 
         for variable, _ in primals.items():
             if not isinstance(variable, BaseVariable):
@@ -303,6 +307,9 @@ class Solution:
         self.vectorizedPrimals = vectorizedPrimals
         """Whether primal values refer to variables' special vectorizations."""
 
+        self.reportedValue = reportedValue
+        """The objective value of the solution as reported by the solver."""
+
     def _status_of_problem(self, problem):
         """Retrieve the problem's verified solution status.
 
@@ -355,7 +362,7 @@ class Solution:
 
     @property
     def value(self):
-        """The objective value of the solution.
+        """The objective value of the solution as computed by PICOS.
 
         .. warning::
 
@@ -375,6 +382,11 @@ class Solution:
 
         return problemCopy.value
 
+    @property
+    def reported_value(self):
+        """The objective value as reported by the solver, or :obj:`None`."""
+        return self.reportedValue
+
     def __str__(self):
         verifiedStatus = self.status
         lastStatus     = self.lastStatus
@@ -446,20 +458,22 @@ class Solution:
                 try:
                     for variable, primal in self.primals.items():
                         thePrimals[toProblem.variables[variable.name]] = primal
-                except KeyError:
+                except KeyError as error:
                     raise RuntimeError(
                         "Cannot apply solution to specified problem as not all "
-                        "variables for which primal values exist were found.")
+                        "variables for which primal values exist were found.") \
+                        from error
 
             if duals:
                 theDuals = {}
                 try:
                     for constraint, dual in self.duals.items():
                         theDuals[toProblem.constraints[constraint.id]] = dual
-                except KeyError:
+                except KeyError as error:
                     raise RuntimeError(
                         "Cannot apply solution to specified problem as not all "
-                        "constraints for which dual values exist were found.")
+                        "constraints for which dual values exist were found.") \
+                        from error
         else:
             thePrimals = self.primals
             theDuals   = self.duals
