@@ -1,22 +1,19 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2020 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Implementation of :class:`Dualization`."""
@@ -44,8 +41,7 @@ _API_START = api_start(globals())
 class Dualization(Reformulation):
     """Lagrange dual problem reformulation."""
 
-    SUPPORTED = Specification.compile(
-        directions=Specification.ALL,
+    SUPPORTED = Specification(
         objectives=[
             AffineExpression],
         variables=CONTINUOUS_VARTYPES,
@@ -58,7 +54,7 @@ class Dualization(Reformulation):
     @classmethod
     def supports(cls, footprint):
         """Implement :meth:`~.reformulation.Reformulation.supports`."""
-        return footprint.options.dualize and footprint << cls.SUPPORTED
+        return footprint.options.dualize and footprint in cls.SUPPORTED
 
     @classmethod
     def predict(cls, footprint):
@@ -165,7 +161,7 @@ class Dualization(Reformulation):
         dual_objective = primal_objective.cst
 
         # Start building the dual linear equalities for each primal variable.
-        obj_coefs = primal_objective.lin._coefs
+        obj_coefs = primal_objective._linear_coefs
         dual_equality_terms = {var: -Constant(obj_coefs[var].T)
             if var in obj_coefs else AffineExpression.zero(var.dim)
             for var in P.variables.values()}
@@ -208,11 +204,11 @@ class Dualization(Reformulation):
                     D.add_constraint(dual_var >= 0)
 
                 # Extend the dual's linear equalities for each primal variable.
-                for var, coef in fx._coefs.items():
+                for var, coef in fx._linear_coefs.items():
                     dual_equality_terms[var] += coef.T*dual_var
 
                 # Extend the dual's objective function.
-                dual_objective -= fx._const.T*dual_var
+                dual_objective -= fx._constant_coef.T*dual_var
 
             elif isinstance(primal_con, SOCConstraint):
                 # Transform constraint into conic standard form.
@@ -226,11 +222,11 @@ class Dualization(Reformulation):
                 D.add_constraint(dual_var << soc())
 
                 # Extend the dual's linear equalities for each primal variable.
-                for var, coef in fx._coefs.items():
+                for var, coef in fx._linear_coefs.items():
                     dual_equality_terms[var] += coef.T*dual_var
 
                 # Extend the dual's objective function.
-                dual_objective -= fx._const.T*dual_var
+                dual_objective -= fx._constant_coef.T*dual_var
             elif isinstance(primal_con, RSOCConstraint):
                 # Transform constraint into conic standard form.
                 fx = (primal_con.ub1 // primal_con.ub2 // primal_con.ne.vec)
@@ -243,11 +239,11 @@ class Dualization(Reformulation):
                 D.add_constraint(dual_var << rsoc(4))
 
                 # Extend the dual's linear equalities for each primal variable.
-                for var, coef in fx._coefs.items():
+                for var, coef in fx._linear_coefs.items():
                     dual_equality_terms[var] += coef.T*dual_var
 
                 # Extend the dual's objective function.
-                dual_objective -= fx._const.T*dual_var
+                dual_objective -= fx._constant_coef.T*dual_var
             elif isinstance(primal_con, LMIConstraint):
                 # Transform constraint into conic standard form.
                 fx = primal_con.psd
@@ -260,11 +256,11 @@ class Dualization(Reformulation):
                 D.add_constraint(dual_var >> 0)
 
                 # Extend the dual's linear equalities for each primal variable.
-                for var, coef in fx._coefs.items():
+                for var, coef in fx._linear_coefs.items():
                     dual_equality_terms[var] += coef.T*dual_var.vec
 
                 # Extend the dual's objective function.
-                dual_objective -= fx._const.T*dual_var.vec
+                dual_objective -= fx._constant_coef.T*dual_var.vec
             else:
                 assert False, "Unexpected constraint type."
 
