@@ -1,22 +1,19 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2018-2019 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Implementation of :class:`CPLEXSolver`."""
@@ -27,9 +24,8 @@ from collections import namedtuple
 import cvxopt
 
 from ..apidoc import api_end, api_start
-from ..compat import assert_import_exists
 from ..constraints import (AffineConstraint, ConvexQuadraticConstraint,
-                           RSOCConstraint, SOCConstraint)
+                           DummyConstraint, RSOCConstraint, SOCConstraint)
 from ..expressions import (CONTINUOUS_VARTYPES, AffineExpression,
                            BinaryVariable, IntegerVariable,
                            QuadraticExpression)
@@ -110,28 +106,38 @@ class CPLEXSolver(Solver):
     # TODO: Allow nonconvex quadratic constraints in the integer case?
     # NOTE: When making changes, also see the section in _solve that tells CPLEX
     #       the problem type.
-    SUPPORTED = Specification.compile(
-        directions=Specification.ALL,
+    SUPPORTED = Specification(
         objectives=[
             AffineExpression,
             QuadraticExpression],
-        variables=Specification.ALL,
         constraints=[
+            DummyConstraint,
             AffineConstraint,
             SOCConstraint,
             RSOCConstraint,
             ConvexQuadraticConstraint])
 
     @classmethod
-    def supports(cls, footprint):
+    def supports(cls, footprint, explain=False):
         """Implement :meth:`~.solver.Solver.supports`."""
-        if not Solver.supports(footprint):
-            return False
+        result = Solver.supports(footprint, explain)
+        if not result or (explain and not result[0]):
+            return result
 
         if footprint.nonconvex_quadratic_objective and footprint.continuous:
-            return False
+            if explain:
+                return (False,
+                    "Continuous problems with nonconvex quadratic objectives.")
+            else:
+                return False
 
-        return footprint << cls.SUPPORTED
+        if footprint not in cls.SUPPORTED:
+            if explain:
+                return False, cls.SUPPORTED.mismatch_reason(footprint)
+            else:
+                return False
+
+        return (True, None) if explain else True
 
     @classmethod
     def default_penalty(cls):
@@ -141,13 +147,18 @@ class CPLEXSolver(Solver):
     @classmethod
     def test_availability(cls):
         """Implement :meth:`~.solver.Solver.test_availability`."""
-        assert_import_exists("cplex")
+        cls.check_import("cplex")
 
     @classmethod
     def names(cls):
         """Implement :meth:`~.solver.Solver.names`."""
         return "cplex", "CPLEX", "IBM ILOG CPLEX Optimization Studio"
 
+    @classmethod
+    def is_free(cls):
+        """Implement :meth:`~.solver.Solver.is_free`."""
+        return False
+
     CplexSOCC = namedtuple("CplexSOCC",
         ("LHSVars", "RHSVar", "LHSCons", "RHSCon", "quadCon"))
 
@@ -292,7 +303,7 @@ class CPLEXSolver(Solver):
 
         cplexI, cplexJ, cplexV = [], [], []
         for (picosVar1, picosVar2), picosCoefficients \
-                in picosExpression.quadratic_forms.items():
+                in picosExpression._sparse_quads.items():
             for sparseIndex in range(len(picosCoefficients)):
                 localVar1Index = picosCoefficients.I[sparseIndex]
                 localVar2Index = picosCoefficients.J[sparseIndex]
@@ -557,7 +568,9 @@ class CPLEXSolver(Solver):
             self._cplexRSOCC[picosConstraint] = \
                 self._import_rscone_constraint(picosConstraint)
         else:
-            assert False, "Constraint type belongs to unsupported problem type."
+            assert isinstance(picosConstraint, DummyConstraint), \
+                "Unexpected constraint type: {}".format(
+                picosConstraint.__class__.__name__)
 
     def _remove_constraint(self, picosConstraint):
         if isinstance(picosConstraint, AffineConstraint):
@@ -577,13 +590,24 @@ class CPLEXSolver(Solver):
             self.int.quadratic_constraints.delete(c.cplexQuadCon)
             self.int.variables.delete(c.cplexLHSVars + c.cplexRHSVars)
         else:
-            assert False, "Constraint type belongs to unsupported problem type."
+            assert isinstance(picosConstraint, DummyConstraint), \
+                "Unexpected constraint type: {}".format(
+                picosConstraint.__class__.__name__)
 
     def _import_affine_objective(self, picosExpression):
         assert isinstance(picosExpression, AffineExpression)
 
+        if picosExpression._constant_coef:
+            offset = picosExpression._constant_coef[0]
+
+            self.int.objective.set_offset(offset)
+
+            if self._debug():
+                self._debug("Constant part of objective imported: {} → {}"
+                    .format(picosExpression.cst.string, offset))
+
         cplexExpression = []
-        for picosVar, picosCoefficient in picosExpression._coefs.items():
+        for picosVar, picosCoefficient in picosExpression._linear_coefs.items():
             assert picosCoefficient.size[0] == 1
 
             for localIndex in range(picosVar.dim):
@@ -598,10 +622,12 @@ class CPLEXSolver(Solver):
             self.int.objective.set_linear(cplexExpression)
 
             if self._debug():
-                self._debug("Affine (part of) objective imported: {} → {}"
-                    .format(picosExpression, cplexExpression))
+                self._debug("Linear part of objective imported: {} → {}"
+                    .format(picosExpression.lin.string, cplexExpression))
 
     def _reset_affine_objective(self):
+        self.int.objective.set_offset(0.0)
+
         linear = self.int.objective.get_linear()
         if any(linear):
             self.int.objective.set_linear(
@@ -621,6 +647,9 @@ class CPLEXSolver(Solver):
             [2.0 * coefficient for coefficient in cplexQuadExpression.val])
         self.int.objective.set_quadratic_coefficients(cplexQuadCoefficients)
 
+        self._debug("Quadratic part of objective imported: {} → {}"
+            .format(picosExpression.quad.string, cplexQuadCoefficients))
+
     def _reset_quadratic_objective(self):
         quadratics = self.int.objective.get_quadratic()
         if quadratics:
@@ -818,17 +847,16 @@ class CPLEXSolver(Solver):
                     self.int.MIP_starts.effort_level.repair)
 
         # Handle CPLEX-specific options.
-        for option, value in o.cplex_params.items():
+        for key, value in o.cplex_params.items():
+            try:
+                parameter = getattr(self.int.parameters, key)
+            except AttributeError as error:
+                self._handle_bad_solver_specific_option_key(key, error)
+
             try:
-                parameter = eval("self.int.parameters." + option)
                 parameter.set(value)
-            except AttributeError:
-                self._handle_bad_option_value("cplex_params",
-                    "CPLEX option '{}' does not exist.".format(option))
-            except cplex.exceptions.errors.CplexError:
-                self._handle_bad_option_value("cplex_params",
-                    "Invalid value '{}' for CPLEX option '{}'."
-                    .format(value, option))
+            except cplex.exceptions.errors.CplexError as error:
+                self._handle_bad_solver_specific_option_value(key, value, error)
 
         # Handle options "cplex_upr_bnd_limit", "cplex_lwr_bnd_limit" and
         # "cplex_bnd_monitor" via a CPLEX callback handler.
@@ -869,16 +897,17 @@ class CPLEXSolver(Solver):
         # This seems necessary, as otherwise LP can get solved as MIP, producing
         # misleading status output (e.g. "not integer feasible").
         conTypes = set(c.__class__ for c in self.ext.constraints.values())
+        quadObj = isinstance(self.ext.no.function, QuadraticExpression)
         cplexTypes = self.int.problem_type
 
-        if isinstance(self.ext.no.function, QuadraticExpression):
-            if conTypes.issubset(set([AffineConstraint])):
+        if quadObj:
+            if conTypes.issubset(set([DummyConstraint, AffineConstraint])):
                 cplexType = cplexTypes.QP if continuous else cplexTypes.MIQP
             else:
                 # Assume quadratic constraint types.
                 cplexType = cplexTypes.QCP if continuous else cplexTypes.MIQCP
         else:
-            if conTypes.issubset(set([AffineConstraint])):
+            if conTypes.issubset(set([DummyConstraint, AffineConstraint])):
                 cplexType = cplexTypes.LP if continuous else cplexTypes.MILP
             else:
                 # Assume quadratic constraint types.
@@ -931,6 +960,11 @@ class CPLEXSolver(Solver):
                 assert maxNumSolutions == 1
 
                 for picosCon in self.ext.constraints.values():
+                    if isinstance(picosCon, DummyConstraint):
+                        duals[picosCon] = cvxopt.spmatrix(
+                            [], [], [], picosCon.size)
+                        continue
+
                     try:
                         if isinstance(picosCon, AffineConstraint):
                             cplexCons = self._cplexLinConNames[picosCon]
@@ -968,15 +1002,19 @@ class CPLEXSolver(Solver):
                     else:
                         duals[picosCon] = picosDual
 
-            # # Retrieve objective value.
-            # try:
-            #     if maxNumSolutions > 1:
-            #         value = self.int.solution.pool.get_objective_value(
-            #             solutionNum)
-            #     else:
-            #         value = self.int.solution.get_objective_value()
-            # except cplex.exceptions.errors.CplexSolverError:
-            #     value = None
+            # Retrieve objective value.
+            try:
+                if quadObj:
+                    # FIXME: Retrieval of QP and MIQP objective value appears to
+                    #        miss the quadratic part.
+                    value = None
+                elif maxNumSolutions > 1:
+                    value = self.int.solution.pool.get_objective_value(
+                        solutionNum)
+                else:
+                    value = self.int.solution.get_objective_value()
+            except cplex.exceptions.errors.CplexSolverError:
+                value = None
 
             # Retrieve solution status.
             code = self.int.solution.get_status()
@@ -991,8 +1029,8 @@ class CPLEXSolver(Solver):
             if o.cplex_bnd_monitor:
                 info["bounds_monitor"] = callback.bounds
 
-            solutions.append(self._make_solution(
-                primals, duals, prmlStatus, dualStatus, probStatus, info))
+            solutions.append(self._make_solution(value, primals, duals,
+                prmlStatus, dualStatus, probStatus, info))
 
         if maxNumSolutions > 1:
             return solutions
