@@ -1,23 +1,20 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2012-2017 Guillaume Sagnol
 # Copyright (C) 2017-2019 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Implementation of :class:`CVXOPTSolver`."""
@@ -26,7 +23,7 @@ import cvxopt
 import numpy
 
 from ..apidoc import api_end, api_start
-from ..constraints import (AffineConstraint, LMIConstraint,
+from ..constraints import (AffineConstraint, DummyConstraint, LMIConstraint,
                            LogSumExpConstraint, RSOCConstraint, SOCConstraint)
 from ..expressions import CONTINUOUS_VARTYPES, AffineExpression, LogSumExp
 from ..modeling.footprint import Specification
@@ -46,13 +43,13 @@ class CVXOPTSolver(Solver):
     :class:`SMCP solver <picos.solvers.solver_smcp.SMCPSolver>`.
     """
 
-    SUPPORTED = Specification.compile(
-        directions=Specification.ALL,
+    SUPPORTED = Specification(
         objectives=[
             AffineExpression,
             LogSumExp],
         variables=CONTINUOUS_VARTYPES,
         constraints=[
+            DummyConstraint,
             AffineConstraint,
             SOCConstraint,
             RSOCConstraint,
@@ -60,12 +57,19 @@ class CVXOPTSolver(Solver):
             LogSumExpConstraint])
 
     @classmethod
-    def supports(cls, footprint):
+    def supports(cls, footprint, explain=False):
         """Implement :meth:`~.solver.Solver.supports`."""
-        if not Solver.supports(footprint):
-            return False
+        result = Solver.supports(footprint, explain)
+        if not result or (explain and not result[0]):
+            return result
 
-        return footprint << cls.SUPPORTED
+        if footprint not in cls.SUPPORTED:
+            if explain:
+                return False, cls.SUPPORTED.mismatch_reason(footprint)
+            else:
+                return False
+
+        return (True, None) if explain else True
 
     @classmethod
     def default_penalty(cls):
@@ -83,6 +87,11 @@ class CVXOPTSolver(Solver):
         """Implement :meth:`~.solver.Solver.names`."""
         return "cvxopt", "CVXOPT", "Python Convex Optimization Solver"
 
+    @classmethod
+    def is_free(cls):
+        """Implement :meth:`~.solver.Solver.is_free`."""
+        return True
+
     @property
     def is_smcp(self):
         """Whether to implement SMCP instead of CVXOPT."""
@@ -131,7 +140,7 @@ class CVXOPTSolver(Solver):
 
         # Construct G.
         I, J, V = [], [], []
-        for variable, coefficients in expression._coefs.items():
+        for variable, coefficients in expression._linear_coefs.items():
             cvxoptVarOffset = self._cvxoptVarOffset[variable]
 
             if not isinstance(coefficients, cvxopt.spmatrix):
@@ -144,7 +153,7 @@ class CVXOPTSolver(Solver):
 
         # Construct h.
         # TODO: Check if making a copy is necessary.
-        h = cvxopt.matrix(expression._const, tc="d")
+        h = cvxopt.matrix(expression._constant_coef, tc="d")
 
         return G, h
 
@@ -262,7 +271,9 @@ class CVXOPTSolver(Solver):
         elif isinstance(constraint, LogSumExpConstraint):
             self._import_lse_constraint(constraint)
         else:
-            assert False, "Unexpected constraint type."
+            assert isinstance(constraint, DummyConstraint), \
+                "Unexpected constraint type: {}".format(
+                constraint.__class__.__name__)
 
     def _import_problem(self):
         self._numVars = sum(var.dim for var in self.ext.variables.values())
@@ -530,6 +541,11 @@ class CVXOPTSolver(Solver):
                 soleq = result["y"]
 
             for constraint in self.ext.constraints.values():
+                if isinstance(constraint, DummyConstraint):
+                    duals[constraint] = cvxopt.spmatrix(
+                        [], [], [], constraint.size)
+                    continue
+
                 dual   = None
                 consSz = len(constraint)
 
@@ -577,6 +593,31 @@ class CVXOPTSolver(Solver):
 
                 duals[constraint] = dual
 
+        # Retrieve objective value.
+        if result is None:
+            value = None
+        elif isGP:
+            value = None
+        else:
+            p = result['primal objective']
+            d = result['dual objective']
+
+            if p is not None and d is not None:
+                value = 0.5 * (p + d)
+            elif p is not None:
+                value = p
+            elif d is not None:
+                value = d
+            else:
+                value = None
+
+        if value is not None and self.ext.no.direction == "max":
+            value = -value
+
+        if self.is_smcp:
+            value = -value
+
+        # Retrieve solution status.
         status = result["status"] if result else "unknown"
         if   status == "optimal":
             primalStatus   = SS_OPTIMAL
@@ -595,8 +636,8 @@ class CVXOPTSolver(Solver):
             dualStatus     = SS_UNKNOWN
             problemStatus  = PS_UNKNOWN
 
-        return self._make_solution(primals, duals, primalStatus, dualStatus,
-            problemStatus, {"cvxopt_sol": result})
+        return self._make_solution(value, primals, duals, primalStatus,
+            dualStatus, problemStatus, {"cvxopt_sol": result})
 
 
 # --------------------------------------
