@@ -1,22 +1,19 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2018-2019 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Implementation of :class:`GurobiSolver`."""
@@ -26,17 +23,16 @@ from collections import namedtuple
 import cvxopt
 
 from ..apidoc import api_end, api_start
-from ..compat import assert_import_exists
 from ..constraints import (AffineConstraint, ConvexQuadraticConstraint,
-                           RSOCConstraint, SOCConstraint)
+                           DummyConstraint, RSOCConstraint, SOCConstraint)
 from ..expressions import (CONTINUOUS_VARTYPES, AffineExpression,
                            BinaryVariable, IntegerVariable,
                            QuadraticExpression)
 from ..modeling.footprint import Specification
-from ..modeling.solution import (PS_FEASIBLE, PS_INFEASIBLE, PS_UNBOUNDED,
-                                 PS_UNKNOWN, PS_UNSTABLE, SS_EMPTY,
-                                 SS_FEASIBLE, SS_INFEASIBLE, SS_OPTIMAL,
-                                 SS_PREMATURE, SS_UNKNOWN)
+from ..modeling.solution import (PS_FEASIBLE, PS_INF_OR_UNB, PS_INFEASIBLE,
+                                 PS_UNBOUNDED, PS_UNKNOWN, PS_UNSTABLE,
+                                 SS_EMPTY, SS_FEASIBLE, SS_INFEASIBLE,
+                                 SS_OPTIMAL, SS_PREMATURE, SS_UNKNOWN)
 from .solver import Solver
 
 _API_START = api_start(globals())
@@ -49,28 +45,38 @@ class GurobiSolver(Solver):
     # TODO: Allow nonconvex quadratic constraints in the integer case?
     # TODO: Don't support (conic) quadratic constraints when duals are
     #       requested because their precision is bad and can't be controlled.
-    SUPPORTED = Specification.compile(
-        directions=Specification.ALL,
+    SUPPORTED = Specification(
         objectives=[
             AffineExpression,
             QuadraticExpression],
-        variables=Specification.ALL,
         constraints=[
+            DummyConstraint,
             AffineConstraint,
             SOCConstraint,
             RSOCConstraint,
             ConvexQuadraticConstraint])
 
     @classmethod
-    def supports(cls, footprint):
+    def supports(cls, footprint, explain=False):
         """Implement :meth:`~.solver.Solver.supports`."""
-        if not Solver.supports(footprint):
-            return False
+        result = Solver.supports(footprint, explain)
+        if not result or (explain and not result[0]):
+            return result
 
         if footprint.nonconvex_quadratic_objective and footprint.continuous:
-            return False
+            if explain:
+                return (False,
+                    "Continuous problems with nonconvex quadratic objectives.")
+            else:
+                return False
 
-        return footprint << cls.SUPPORTED
+        if footprint not in cls.SUPPORTED:
+            if explain:
+                return False, cls.SUPPORTED.mismatch_reason(footprint)
+            else:
+                return False
+
+        return (True, None) if explain else True
 
     @classmethod
     def default_penalty(cls):
@@ -80,13 +86,18 @@ class GurobiSolver(Solver):
     @classmethod
     def test_availability(cls):
         """Implement :meth:`~.solver.Solver.test_availability`."""
-        assert_import_exists("gurobipy")
+        cls.check_import("gurobipy")
 
     @classmethod
     def names(cls):
         """Implement :meth:`~.solver.Solver.names`."""
         return "gurobi", "Gurobi", "Gurobi Optimizer"
 
+    @classmethod
+    def is_free(cls):
+        """Implement :meth:`~.solver.Solver.is_free`."""
+        return False
+
     GurobiSOCC = namedtuple("GurobiSOCC",
         ("LHSVars", "RHSVar", "LHSCons", "RHSCon", "quadCon"))
 
@@ -213,7 +224,7 @@ class GurobiSolver(Solver):
         # Import quadratic form.
         gurobiI, gurobiJ, gurobiV = [], [], []
         for (picosVar1, picosVar2), picosCoefficients \
-                in picosExpression.quadratic_forms.items():
+                in picosExpression._sparse_quads.items():
             for sparseIndex in range(len(picosCoefficients)):
                 localVar1Index = picosCoefficients.I[sparseIndex]
                 localVar2Index = picosCoefficients.J[sparseIndex]
@@ -379,7 +390,9 @@ class GurobiSolver(Solver):
             self._gurobiRSOCC[picosConstraint] = \
                 self._import_rscone_constraint(picosConstraint)
         else:
-            assert False, "Constraint type belongs to unsupported problem type."
+            assert isinstance(picosConstraint, DummyConstraint), \
+                "Unexpected constraint type: {}".format(
+                picosConstraint.__class__.__name__)
 
     def _remove_constraint(self, picosConstraint):
         if isinstance(picosConstraint, AffineConstraint):
@@ -397,7 +410,9 @@ class GurobiSolver(Solver):
             self.int.remove(c.gurobiLHSCons + c.gurobiRHSCons
                 + [c.gurobiQuadCon] + c.gurobiLHSVars + c.gurobiRHSVars)
         else:
-            assert False, "Constraint type belongs to unsupported problem type."
+            assert isinstance(picosConstraint, DummyConstraint), \
+                "Unexpected constraint type: {}".format(
+                picosConstraint.__class__.__name__)
 
     def _import_objective(self):
         import gurobipy as gurobi
@@ -424,7 +439,7 @@ class GurobiSolver(Solver):
         import gurobipy as gurobi
 
         # Create a problem instance.
-        if self.ext.options.license_warnings:
+        if self._license_warnings:
             self.int = gurobi.Model()
         else:
             with self._enforced_verbosity():
@@ -534,16 +549,14 @@ class GurobiSolver(Solver):
             self._reset_variable_values()
 
         # Handle Gurobi-specific options.
-        for option, value in self.ext.options.gurobi_params.items():
-            if not self.int.getParamInfo(option):
-                self._handle_bad_option_value("gurobi_params",
-                    "Gurobi option '{}' does not exist.".format(option))
+        for key, value in self.ext.options.gurobi_params.items():
+            if not self.int.getParamInfo(key):
+                self._handle_bad_solver_specific_option_key(key)
+
             try:
-                self.int.setParam(option, value)
+                self.int.setParam(key, value)
             except TypeError as error:
-                self._handle_bad_option_value("gurobi_params",
-                    "Invalid value '{}' for Gurobi option '{}': {}"
-                    .format(value, option, str(error)))
+                self._handle_bad_solver_specific_option_value(key, value, error)
 
         # Handle unsupported options.
         self._handle_unsupported_option("treememory")
@@ -580,14 +593,26 @@ class GurobiSolver(Solver):
         # Retrieve duals.
         duals = {}
         if self.ext.options.duals is not False and self.ext.is_continuous():
+            Pi = gurobi.GRB.Attr.Pi
+
             for picosCon in self.ext.constraints.values():
+                if isinstance(picosCon, DummyConstraint):
+                    duals[picosCon] = cvxopt.spmatrix([], [], [], picosCon.size)
+                    continue
+
+                # HACK: Work around gurobiCon.getAttr(gurobi.GRB.Attr.Pi)
+                #       printing a newline to console when it raises an
+                #       AttributeError and OutputFlag is enabled. This is a
+                #       WONTFIX on Gurobi's end (PICOS #264, Gurobi #14248).
+                oldOutput = self.int.Params.OutputFlag
+                self.int.Params.OutputFlag = 0
+
                 try:
                     if isinstance(picosCon, AffineConstraint):
                         gurobiCons = self._gurobiLinearConstraints[picosCon]
                         gurobiDuals = []
                         for gurobiCon in gurobiCons:
-                            gurobiDuals.append(
-                                gurobiCon.getAttr(gurobi.GRB.Attr.Pi))
+                            gurobiDuals.append(gurobiCon.getAttr(Pi))
                         picosDual = cvxopt.matrix(gurobiDuals, picosCon.size)
 
                         # Flip sign based on constraint relation.
@@ -595,21 +620,23 @@ class GurobiSolver(Solver):
                             picosDual = -picosDual
                     elif isinstance(picosCon, SOCConstraint):
                         gurobiMetaCon = self._gurobiSOCC[picosCon]
-                        lb = gurobiMetaCon.RHSCon.getAttr(gurobi.GRB.Attr.Pi)
-                        z = [-constraint.getAttr(gurobi.GRB.Attr.Pi)
+                        lb = gurobiMetaCon.RHSCon.getAttr(Pi)
+                        z = [-constraint.getAttr(Pi)
                             for constraint in gurobiMetaCon.LHSCons]
                         picosDual = cvxopt.matrix([lb] + z)
                     elif isinstance(picosCon, RSOCConstraint):
                         gurobiMetaCon = self._gurobiRSOCC[picosCon]
-                        ab = [constraint.getAttr(gurobi.GRB.Attr.Pi)
+                        ab = [constraint.getAttr(Pi)
                             for constraint in gurobiMetaCon.RHSCons]
-                        z = [-constraint.getAttr(gurobi.GRB.Attr.Pi)
+                        z = [-constraint.getAttr(Pi)
                             for constraint in gurobiMetaCon.LHSCons]
                         picosDual = cvxopt.matrix(ab + z)
                     elif isinstance(picosCon, ConvexQuadraticConstraint):
                         picosDual = None
                     else:
-                        assert False, "Unexpected constraint type."
+                        assert isinstance(picosCon, DummyConstraint), \
+                            "Unexpected constraint type: {}".format(
+                            picosCon.__class__.__name__)
 
                     # Flip sign based on objective sense.
                     if picosDual and self.ext.no.direction == "min":
@@ -619,12 +646,18 @@ class GurobiSolver(Solver):
                 else:
                     duals[picosCon] = picosDual
 
-        # # Retrieve objective value.
-        # try:
-        #     objectiveValue = self.int.getAttr(gurobi.GRB.Attr.ObjVal)
-        # except AttributeError:
-        #     objectiveValue = None
+                # HACK: See above. Also: Silence Gurobi while enabling output.
+                if oldOutput != 0:
+                    with self._enforced_verbosity(noStdOutAt=float("inf")):
+                        self.int.Params.OutputFlag = oldOutput
 
+        # Retrieve objective value.
+        try:
+            value = self.int.getAttr(gurobi.GRB.Attr.ObjVal)
+        except AttributeError:
+            value = None
+
+        # Retrieve solution status.
         statusCode = self.int.getAttr(gurobi.GRB.Attr.Status)
         if statusCode   == gurobi.GRB.Status.LOADED:
             raise RuntimeError("Gurobi claims to have just loaded the problem "
@@ -640,7 +673,7 @@ class GurobiSolver(Solver):
         elif statusCode == gurobi.GRB.Status.INF_OR_UNBD:
             primalStatus   = SS_UNKNOWN
             dualStatus     = SS_UNKNOWN
-            problemStatus  = PS_UNKNOWN
+            problemStatus  = PS_INF_OR_UNB
         elif statusCode == gurobi.GRB.Status.UNBOUNDED:
             primalStatus   = SS_UNKNOWN
             dualStatus     = SS_INFEASIBLE
@@ -697,7 +730,7 @@ class GurobiSolver(Solver):
             problemStatus  = PS_UNKNOWN
 
         return self._make_solution(
-            primals, duals, primalStatus, dualStatus, problemStatus)
+            value, primals, duals, primalStatus, dualStatus, problemStatus)
 
 
 # --------------------------------------
