@@ -1,22 +1,19 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2018-2019 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Implementation of :class:`MOSEKSolver`."""
@@ -27,9 +24,9 @@ import sys
 import cvxopt
 
 from ..apidoc import api_end, api_start
-from ..compat import assert_import_exists
 from ..constraints import (AffineConstraint, ConvexQuadraticConstraint,
-                           LMIConstraint, RSOCConstraint, SOCConstraint)
+                           DummyConstraint, LMIConstraint, RSOCConstraint,
+                           SOCConstraint)
 from ..expressions import (AffineExpression, BinaryVariable, IntegerVariable,
                            QuadraticExpression)
 from ..modeling.footprint import Specification
@@ -51,13 +48,12 @@ class MOSEKSolver(Solver):
     the high level Fusion API, which would be the prefered interface otherwise.
     """
 
-    SUPPORTED = Specification.compile(
-        directions=Specification.ALL,
+    SUPPORTED = Specification(
         objectives=[
             AffineExpression,
             QuadraticExpression],
-        variables=Specification.ALL,
         constraints=[
+            DummyConstraint,
             AffineConstraint,
             SOCConstraint,
             RSOCConstraint,
@@ -65,14 +61,18 @@ class MOSEKSolver(Solver):
             LMIConstraint])
 
     @classmethod
-    def supports(cls, footprint):
+    def supports(cls, footprint, explain=False):
         """Implement :meth:`~.solver.Solver.supports`."""
-        if not Solver.supports(footprint):
-            return False
+        result = Solver.supports(footprint, explain)
+        if not result or (explain and not result[0]):
+            return result
 
         # No nonconvex quadratic objectives.
         if footprint.nonconvex_quadratic_objective:
-            return False
+            if explain:
+                return False, "Problems with nonconvex quadratic objectives."
+            else:
+                return False
 
         conic = any(("con", constraint) in footprint
             for constraint in (SOCConstraint, RSOCConstraint, LMIConstraint))
@@ -82,13 +82,26 @@ class MOSEKSolver(Solver):
 
         # No mixing of quadratic and conic problems.
         if conic and quadratic:
-            return False
+            if explain:
+                return (False,
+                    "Problems that mix conic and quadratic constraints.")
+            else:
+                return False
 
         # No integer SDPs.
         if footprint.integer and ("con", LMIConstraint) in footprint:
-            return False
+            if explain:
+                return False, "Integer Semidefinite Programs."
+            else:
+                return False
 
-        return footprint << cls.SUPPORTED
+        if footprint not in cls.SUPPORTED:
+            if explain:
+                return False, cls.SUPPORTED.mismatch_reason(footprint)
+            else:
+                return False
+
+        return (True, None) if explain else True
 
     @classmethod
     def default_penalty(cls):
@@ -98,13 +111,18 @@ class MOSEKSolver(Solver):
     @classmethod
     def test_availability(cls):
         """Implement :meth:`~.solver.Solver.test_availability`."""
-        assert_import_exists("mosek")
+        cls.check_import("mosek")
 
     @classmethod
     def names(cls):
         """Implement :meth:`~.solver.Solver.names`."""
         return "mosek", "MOSEK (Optimizer)", "MOSEK via Optimizer API"
 
+    @classmethod
+    def is_free(cls):
+        """Implement :meth:`~.solver.Solver.is_free`."""
+        return False
+
     def __init__(self, problem):
         """Initialize a MOSEK (Optimizer) solver interface.
 
@@ -224,7 +242,7 @@ class MOSEKSolver(Solver):
         ID, JD, VD = [], [], []
         IU, JU, VU = [], [], []
         for (picosVar1, picosVar2), picosCoefs \
-        in picosExpression.quadratic_forms.items():
+        in picosExpression._sparse_quads.items():
             for sparseIndex in range(len(picosCoefs)):
                 localVar1Index   = picosCoefs.I[sparseIndex]
                 localVar2Index   = picosCoefs.J[sparseIndex]
@@ -494,7 +512,9 @@ class MOSEKSolver(Solver):
         elif isinstance(picosConstraint, LMIConstraint):
             self._import_sdp_constraint(picosConstraint)
         else:
-            assert False, "Constraint type not supported."
+            assert isinstance(picosConstraint, DummyConstraint), \
+                "Unexpected constraint type: {}".format(
+                picosConstraint.__class__.__name__)
 
     def _reset_objective(self):
         # Reset affine part.
@@ -509,7 +529,7 @@ class MOSEKSolver(Solver):
         mosekIndices      = []
         mosekCoefficients = []
 
-        for picosVar, picosCoef in picosObjective._coefs.items():
+        for picosVar, picosCoef in picosObjective._linear_coefs.items():
             for localIndex in range(picosVar.dim):
                 if picosCoef[localIndex]:
                     mosekIndex = self._mosekVarOffset[picosVar] + localIndex
@@ -518,8 +538,8 @@ class MOSEKSolver(Solver):
 
         self.int.putclist(mosekIndices, mosekCoefficients)
 
-        if picosObjective._const:
-            self.int.putcfix(picosObjective._const[0])
+        if picosObjective._constant_coef:
+            self.int.putcfix(picosObjective._constant_coef[0])
 
     def _import_quadratic_objective(self, picosObjective):
         # Import the quadratic part.
@@ -698,11 +718,11 @@ class MOSEKSolver(Solver):
                 self.ext.options.max_fsb_nodes)
 
         # Handle MOSEK-specific options.
-        for option, value in self.ext.options.mosek_params.items():
+        for key, value in self.ext.options.mosek_params.items():
             try:
-                self.int.putparam(option.upper(), str(value))
+                self.int.putparam(key.upper(), str(value))
             except mosek.Error as error:
-                self._handle_bad_option_value("mosek_params", str(error))
+                self._handle_bad_solver_specific_option(key, value, error)
 
         # Handle unsupported options.
         # TODO: Handle "hotstart" option (via mio_construct_sol).
@@ -754,8 +774,14 @@ class MOSEKSolver(Solver):
             minimizing = self.int.getobjsense() == mosek.objsense.minimize
 
             for constraint in self.ext.constraints.values():
+                if isinstance(constraint, DummyConstraint):
+                    duals[constraint] = cvxopt.spmatrix(
+                        [], [], [], constraint.size)
+                    continue
+
                 length = len(constraint)
                 dual   = [float("nan")]*length
+
                 if isinstance(constraint, AffineConstraint):
                     offset = self._mosekLinConOffset[constraint]
                     self.int.getyslice(solType, offset, offset + length, dual)
@@ -808,6 +834,10 @@ class MOSEKSolver(Solver):
 
                 duals[constraint] = dual
 
+        # Retrieve objective value.
+        value = self.int.getprimalobj(solType)
+
+        # Retrieve solution status.
         primalStatus  = self._solution_status_pic2msk(
             self.int.getsolsta(solType), primalSolution=True)
         dualStatus    = self._solution_status_pic2msk(
@@ -816,7 +846,7 @@ class MOSEKSolver(Solver):
             self.int.getprosta(solType), not self.ext.is_continuous())
 
         return self._make_solution(
-            primals, duals, primalStatus, dualStatus, problemStatus)
+            value, primals, duals, primalStatus, dualStatus, problemStatus)
 
     def _solution_status_pic2msk(self, statusCode, primalSolution):
         from mosek import solsta as ss
