@@ -1,22 +1,19 @@
-# coding: utf-8
-
 # ------------------------------------------------------------------------------
 # Copyright (C) 2017-2019 Maximilian Stahlberg
 #
 # This file is part of PICOS.
 #
-# PICOS is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# PICOS is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# PICOS is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# PICOS is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# this program.  If not, see <http://www.gnu.org/licenses/>.
 # ------------------------------------------------------------------------------
 
 """Implementation of :class:`MOSEKFusionSolver`."""
@@ -26,9 +23,8 @@ import sys
 import cvxopt
 
 from ..apidoc import api_end, api_start
-from ..compat import assert_import_exists
-from ..constraints import (AffineConstraint, LMIConstraint, RSOCConstraint,
-                           SOCConstraint)
+from ..constraints import (AffineConstraint, DummyConstraint, LMIConstraint,
+                           RSOCConstraint, SOCConstraint)
 from ..expressions import AffineExpression, BinaryVariable, IntegerVariable
 from ..modeling.footprint import Specification
 from ..modeling.solution import (PS_FEASIBLE, PS_ILLPOSED, PS_INF_OR_UNB,
@@ -51,28 +47,37 @@ class MOSEKFusionSolver(Solver):
     interface.
     """
 
-    SUPPORTED = Specification.compile(
-        directions=Specification.ALL,
+    SUPPORTED = Specification(
         objectives=[
             AffineExpression],
-        variables=Specification.ALL,
         constraints=[
+            DummyConstraint,
             AffineConstraint,
             SOCConstraint,
             RSOCConstraint,
             LMIConstraint])
 
     @classmethod
-    def supports(cls, footprint):
+    def supports(cls, footprint, explain=False):
         """Implement :meth:`~.solver.Solver.supports`."""
-        if not Solver.supports(footprint):
-            return False
+        result = Solver.supports(footprint, explain)
+        if not result or (explain and not result[0]):
+            return result
 
         # No integer SDPs.
         if footprint.integer and ("con", LMIConstraint) in footprint:
-            return False
+            if explain:
+                return False, "Integer Semidefinite Programs."
+            else:
+                return False
 
-        return footprint << cls.SUPPORTED
+        if footprint not in cls.SUPPORTED:
+            if explain:
+                return False, cls.SUPPORTED.mismatch_reason(footprint)
+            else:
+                return False
+
+        return (True, None) if explain else True
 
     @classmethod
     def default_penalty(cls):
@@ -82,13 +87,18 @@ class MOSEKFusionSolver(Solver):
     @classmethod
     def test_availability(cls):
         """Implement :meth:`~.solver.Solver.test_availability`."""
-        assert_import_exists("mosek.fusion")
+        cls.check_import("mosek.fusion")
 
     @classmethod
     def names(cls):
         """Implement :meth:`~.solver.Solver.names`."""
         return "mskfsn", "MOSEK (Fusion)", "MOSEK via Fusion API"
 
+    @classmethod
+    def is_free(cls):
+        """Implement :meth:`~.solver.Solver.is_free`."""
+        return False
+
     def __init__(self, problem):
         """Initialize a MOSEK (Fusion) solver interface.
 
@@ -189,7 +199,7 @@ class MOSEKFusionSolver(Solver):
 
         # Convert linear part of expression.
         firstSummand = True
-        for picosVar, factor in picosExpression._coefs.items():
+        for picosVar, factor in picosExpression._linear_coefs.items():
             mosekVar = self.knownVariables[picosVar]
 
             summand = msk.Expr.mul(self._matrix_cvx2msk(factor), mosekVar)
@@ -202,7 +212,7 @@ class MOSEKFusionSolver(Solver):
         # Convert constant term of expression.
         if picosExpression.constant is not None:
             mosekConstant = msk.Expr.constTerm(
-                self._matrix_cvx2msk(picosExpression._const))
+                self._matrix_cvx2msk(picosExpression._constant_coef))
 
             if firstSummand:
                 mosekExpression = mosekConstant
@@ -347,7 +357,7 @@ class MOSEKFusionSolver(Solver):
         # Rewrite constraint in MOSEK types: The linear function is represented
         # as a MOSEK expression while the constant term becomes a MOSEK domain.
         linear = self._affinexp_pic2msk(linear[:])
-        bound  = self._matrix_cvx2msk(bound._const)
+        bound  = self._matrix_cvx2msk(bound._constant_coef)
 
         if picosCon.is_increasing():
             domain = msk.Domain.lessThan(bound)
@@ -422,7 +432,8 @@ class MOSEKFusionSolver(Solver):
             elif isinstance(picosCon, LMIConstraint):
                 mosekCon = self._import_sdp_constraint(picosCon)
             else:
-                assert False, "Constraint type not supported."
+                assert False, "Unexpected constraint type: {}".format(
+                    picosCon.__class__.__name__)
 
             self.int.setLogHandler(sys.stdout)
 
@@ -469,7 +480,8 @@ class MOSEKFusionSolver(Solver):
 
         # Import constraints.
         for constraint in self.ext.constraints.values():
-            self._import_constraint(constraint)
+            if not isinstance(constraint, DummyConstraint):
+                self._import_constraint(constraint)
 
         # Set objective.
         self._import_objective()
@@ -616,16 +628,13 @@ class MOSEKFusionSolver(Solver):
             self._import_variable_values(integralOnly=True)
 
         # Handle MOSEK-specific options.
-        for option, value in self.ext.options.mskfsn_params.items():
+        for key, value in self.ext.options.mskfsn_params.items():
             try:
-                self.int.setSolverParam(option, value)
-            except msk.ParameterError:
-                self._handle_bad_option_value("mskfsn_params",
-                    "MOSEK option '{}' does not exist.".format(option))
+                self.int.setSolverParam(key, value)
+            except msk.ParameterError as error:
+                self._handle_bad_solver_specific_option_key(key, error)
             except ValueError as error:
-                self._handle_bad_option_value("mskfsn_params",
-                    "Invalid value '{}' for MOSEK option '{}': {}"
-                    .format(value, option, str(error)))
+                self._handle_bad_solver_specific_option_value(key, value, error)
 
         # Handle unsupported options.
         self._handle_unsupported_option("treememory")
@@ -648,6 +657,10 @@ class MOSEKFusionSolver(Solver):
         duals = {}
         if self.ext.options.duals is not False:
             for picosCon in self.ext.constraints.values():
+                if isinstance(picosCon, DummyConstraint):
+                    duals[picosCon] = cvxopt.spmatrix([], [], [], picosCon.size)
+                    continue
+
                 # Retrieve corresponding MOSEK constraint.
                 mosekCon = self.knownConstraints[picosCon]
 
@@ -705,12 +718,13 @@ class MOSEKFusionSolver(Solver):
 
                 duals[picosCon] = picosDual
 
-        # # Retrieve objective value.
-        # try:
-        #     objectiveValue = float(self.int.primalObjValue())
-        # except msk.SolutionError:
-        #     objectiveValue = None
+        # Retrieve objective value.
+        try:
+            value = float(self.int.primalObjValue())
+        except msk.SolutionError:
+            value = None
 
+        # Retrieve solution status.
         primalStatus  = self._solution_status_pic2msk(
             self.int.getPrimalSolutionStatus())
         dualStatus    = self._solution_status_pic2msk(
@@ -725,7 +739,7 @@ class MOSEKFusionSolver(Solver):
             dualStatus = SS_INFEASIBLE
 
         return self._make_solution(
-            primals, duals, primalStatus, dualStatus, problemStatus)
+            value, primals, duals, primalStatus, dualStatus, problemStatus)
 
     def _solution_status_pic2msk(self, statusCode):
         from mosek.fusion import SolutionStatus as ss
